/*
 * Copyright 2010 Samit Badle, Samit.Badle@gmail.com
 * http://blog.reallysimplethoughts.com/tag/selenium-expert-selenium-ide-plug-in/
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

function SeleniumExpert(editor) {
  this.editor = editor;
  this.commandInspectors = new Array();
  this.view = document.getElementById("seExpertView");
  this.viewDoc = this.view.contentDocument;
  this.categoriesElement = this.viewDoc.getElementById("categories");
  this.msgElement = this.viewDoc.getElementById("msg");
  this.splashElement = this.viewDoc.getElementById("splash");
  this.viewDoc.getElementById("bloglink").addEventListener("click", function() {
    openTabOrWindow('http://links.reallysimplethoughts.com/go/selenium-expert-news');
  }, false);
  this.subclassTreeView = new SeleniumExpert.Subclass();
  this.subclassTreeView.chainMethod(window.TreeView.prototype, 'getCellProperties', 'sbseGetCellProperties', this, this._treeView_getCellProperties);
  this.observer = new SeleniumExpert.SuiteObserver(editor, this);
}

SeleniumExpert.DEFAULT_MESSAGE = "Selenium Expert by Samit Badle. Click the inspect button to run inspections.";
SeleniumExpert.HTML_FORMAT_ONLY = "Selenium Expert can inspect Selenese. Switch the format to HTML first.";


/**
 * The main public interface
 */
SeleniumExpert.prototype.registerCommandInspector = function(inspector, pattern) {
  //Send a null pattern or skip the pattern argument to receive all commands
  if (!pattern) {
    pattern = ".";
  }
  this.commandInspectors.push({ 'inspector': inspector, 'pattern': new RegExp(pattern), 'results': new Array() });
};

SeleniumExpert.prototype.inspect = function() {
  this.clear();
  this.splashElement.className = 'no-msg';
  if (this.editor.app.getCurrentFormat().name == "HTML") {
    var result = this._applyToCommands(this.inspectCommand, 'Inspecting', 'Inspected');
    this.updatePane();
    this.editor.treeView.refresh();
    return true;
  } else {
    this.showMessage(SeleniumExpert.HTML_FORMAT_ONLY);
  }
  return false;
};

SeleniumExpert.prototype.inspectCommand = function(cmd, testCase) {
  var processedCount = 0;
  delete cmd.inspections;
  for (var i = 0; i < this.commandInspectors.length; i++) {
    if (!this.commandInspectors[i].pattern || this.commandInspectors[i].pattern.test(cmd.command)) {
      var result = this.commandInspectors[i].inspector.inspect(cmd);
      if (result) {
        processedCount++;
        if (!cmd.inspections) {
          cmd.inspections = new Array();
        }
        cmd.inspections.push({'inspector': this.commandInspectors[i].inspector, 'result': result});
        this.commandInspectors[i].results.push({'command': cmd, 'result': result, 'testCase': testCase})
      }
    }
  }
  return processedCount;
};

SeleniumExpert.prototype.fix = function() {
  this.splashElement.className = 'no-msg';
  //TODO: backup first
  if (this.editor.app.getCurrentFormat().name == "HTML") {
    var backupFolder = this.backupSuite();
    if (backupFolder) {
      var processedCount = 0;
      var inspectorsFixed = 0;
      for (var i = 0; i < this.commandInspectors.length; i++) {
        if (this.commandInspectors[i].inspector.fix) {
          var checkbox = this.viewDoc.getElementById("cat" + i + "-check");
          if (checkbox && checkbox.checked) {
            inspectorsFixed++;
            processedCount += this.fixForInspector(checkbox.getAttribute("value"));
          }
        }
      }
      this.showMessage("Fixed " + processedCount + " command inspections for " + inspectorsFixed + " categories. Test suite before changes is backed up to '" + backupFolder + "'.");
      this.updatePane();
      this.editor.treeView.refresh();
      this.editor.suiteTreeView.refresh();
      return true;
    }
  } else {
    this.showMessage(SeleniumExpert.HTML_FORMAT_ONLY);
  }
  return false;
};

SeleniumExpert.prototype.fixForInspector = function(inspectorId) {
  var processedCount = 0;
  var inspector = this.commandInspectors[inspectorId].inspector;
  if (inspector.fix) {
    var results = this.commandInspectors[inspectorId].results;
    for (var i = 0; i < results.length; i++) {
      if (this.fixSingleInspection(inspector, results[i])) {
        processedCount++;
      }
    }
  }
  return processedCount;
};

SeleniumExpert.prototype.fixSingleInspection = function(inspector, inspectionResult) {
  if (inspector.fix) {
    var fixed = inspector.fix(inspectionResult.command);
    if (fixed) {
      inspectionResult.fixed = fixed[0];
      //TODO: Currently only supporting same command fixes, multi command and remove command fixes are not supported
      var cmd = inspectionResult.command;
      if (cmd.inspections) {
        //Search for the inspection result in the command to update it as fixed
        for (var c = 0; c < cmd.inspections.length; c++) {
          if (cmd.inspections[c].inspector == inspector) {
            cmd.inspections[c].fixed = fixed[0];
            break;
          }
        }
      } else {
        //TODO: how did the inspections disappear from the command?
      }
      //Flag the command as having some fixed inspections
      cmd.hasFixedInspections = true;
      inspectionResult.testCase.content.setModified();
      return fixed;
    }
  }
  return null;
};

SeleniumExpert.prototype.clear = function() {
  for (var i = 0; i < this.commandInspectors.length; i++) {
    this.commandInspectors[i].results = new Array();
  }
  var curSuite = this.editor.app.getTestSuite();
  if (curSuite) {
    for (var t = 0; t < curSuite.tests.length; t++) {
      if (curSuite.tests[t].content) {
        var curCase = curSuite.tests[t].content;
        for (var c = 0; c < curCase.commands.length; c++) {
          var cmd = curCase.commands[c];
          if (cmd.inspections) {
            delete cmd.inspections;
          }
          if (cmd.hasFixedInspections) {
            delete cmd.hasFixedInspections;
          }
        }
      }
    }
  }
  this.clearPane();
  this.editor.treeView.refresh();
  this.showMessage(SeleniumExpert.DEFAULT_MESSAGE);
  this.splashElement.className = 'msg';
};

/*
 SeleniumExpert.prototype.showCommand = function(InspectorId, InspectionID) {
 var result = this.commandInspectors[InspectorId].results[InspectionID];
 var curSuite = this.editor.app.getTestSuite();
 if (curSuite) {
 for (var t = 0; t < curSuite.tests.length; t++) {
 if (curSuite.tests[t] == result.testCase) {
 this.editor.suiteTreeView.scrollToRow(t);
 this.editor.app.showTestCaseFromSuite(result.testCase);
 var cmds = result.testCase.content.commands;
 for (var i = 0; i < cmds.length; i++) {
 if (cmds[i] == result.command) {
 this.editor.treeView.tree.view.selection.select(i);
 this.editor.infoPanel.switchView(this.editor.seExpertView);
 //Phew!
 }
 }
 }
 }
 }
 };*/


/**
 * The internal utility fns
 */
SeleniumExpert.prototype._applyToCommands = function(commandProcessor, processingMsg, processedMsg) {
  var testCaseCount = 0;
  var processedTestCaseCount = 0;
  var commandCount = 0;
  var processedCount = 0;

  var curSuite = this.editor.app.getTestSuite();
  if (curSuite) {
    testCaseCount = curSuite.tests.length;
    this.clearPane();
    if (processingMsg) {
      this.showMessage(processingMsg + " " + curSuite.tests.length + " test cases in the current test suite.");
    }
    for (var t = 0; t < curSuite.tests.length; t++) {
      this.loadTestCase(curSuite.tests[t]);
      if (curSuite.tests[t].content) {
        processedTestCaseCount++;
        var curCase = curSuite.tests[t].content;
        commandCount += curCase.commands.length;
        for (var c = 0; c < curCase.commands.length; c++) {
          var cmd = curCase.commands[c];
          if (cmd.type == "command") {
            processedCount += commandProcessor.call(this, cmd, curSuite.tests[t]);
          }
        }
      }
    }
    if (processedMsg) {
      this.showMessage(processedMsg + " " + curSuite.tests.length + " test cases and " + commandCount + " commands. Click on Fix to apply changes. Backup everything first!");
    }
  } else {
    //TODO: Why is there no current suite???
  }
  return {'testCases': testCaseCount, 'processedTestCases': processedTestCaseCount, 'commands': commandCount, 'processed': processedCount}
};

SeleniumExpert.prototype.loadTestCase = function(testCase) {
  if (!testCase.content) {
    try {
      var test = this.editor.app.getCurrentFormat().loadFile(testCase.getFile(), false);
      test.title = testCase.getTitle(); // load title from suite
      testCase.content = test;
    } catch(error) {
      //Silently ignore the test case
      return false;
    }
  }
  return true;
};

SeleniumExpert.prototype.updatePane = function() {
  this.categoriesElement.innerHTML = '';
  for (var i = 0; i < this.commandInspectors.length; i++) {
    if (this.commandInspectors[i].results.length > 0) {
      var results = this.commandInspectors[i].results;
      var fixedCount = 0;
      var r;
      for (r = 0; r < results.length; r++) {
        if (results[r].fixed) {
          fixedCount++;
        }
      }
      var summary = "(" + results.length + " items" + (fixedCount > 0 ? ", " + fixedCount + " fixed" : "") + ")"
      var cat = this.insertInspectionCategory(i, this.commandInspectors[i].inspector.description, summary, this.commandInspectors[i].inspector.fix, results.length != fixedCount);
      for (r = 0; r < results.length; r++) {
        this.insertInspectionMsg(cat, i, r, results[r].result, (results[r].fixed ? results[r].fixed : ""));
      }
    }
  }
};

SeleniumExpert.prototype.backupSuite = function() {
  var homeDir = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("Home", Components.interfaces.nsIFile);
  homeDir.append("SeleniumExpertBackups");
  if (!homeDir.exists() || !homeDir.isDirectory()) {   // if it doesn't exist, create
    homeDir.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);
  }
  var date = new Date();
  var ts = date.getFullYear() + "-" + (date.getMonth() + 1) + "-" + (date.getDay() + 1) + "_" + date.getHours() + "-" + date.getMinutes() + "-" + date.getSeconds() + "-" + date.getMilliseconds();
  homeDir.append(ts);
  if (!homeDir.exists() || !homeDir.isDirectory()) {   // if it doesn't exist, create
    homeDir.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);
  }
  var curSuite = this.editor.app.getTestSuite();
  if (curSuite) {
    var htmlFormat = this.getHTMLFormatter();
    var suite = new TestSuite();
    for (var t = 0; t < curSuite.tests.length; t++) {
      this.loadTestCase(curSuite.tests[t]);
      if (curSuite.tests[t].content) {
        var content = curSuite.tests[t].content.createCopy();
        var file = FileUtils.getFile(homeDir.path);
        file.append("TestCase" + t + ".htm");
        htmlFormat.saveAs(content, file.path);
        suite.tests.push(new TestSuite.TestCase(suite, "TestCase" + t + ".htm", curSuite.tests[t].getTitle()));
      }
    }
    suite.file = FileUtils.getFile(homeDir.path);
    suite.file.append("TestSuite.htm");
    suite.save();
    return homeDir.path;
  } else {
    //TODO: Why is there no current suite???
  }

  /*if (confirm("Fix makes changes to your test cases and can cause them to break or get corrupted. You are strongly recommended to backup all files in your test suite first.\nClick OK only if you are willing to accept the risks.")) {
   return true;
   }*/
  return null;
};

SeleniumExpert.prototype.getHTMLFormatter = function() {
  var formats = this.editor.app.getFormats().formats;
  for (var i = 0; i < formats.length; i++) {
    if (formats[i].name.toLowerCase().indexOf('html') >= 0) {
      return formats[i];
    }
  }
  return null;
};

SeleniumExpert.prototype._treeView_getCellProperties = function(baseClassObject, retValue, args) {
  if (0 == args[1].index) {
    var cmd = baseClassObject.getCommand(args[0]);
    if (cmd) {
      if (cmd.hasFixedInspections) {
        args[2].AppendElement(baseClassObject.atomService.getAtom("expertfixed"));
      } else if (cmd.inspections) {
        args[2].AppendElement(baseClassObject.atomService.getAtom("expertinfo"));
      }
    }
  }
  return retValue;
};

SeleniumExpert.Subclass = function() {
};
SeleniumExpert.Subclass.prototype.chainMethod = function(baseClassObject, methodName, overrideMarkerName, subClassObject, chainedMethod) {
  if (baseClassObject) {
    if (!baseClassObject[overrideMarkerName]) {
      this.baseClassMethod = baseClassObject[methodName];
      this.subClassObject = subClassObject;
      this.chainedMethod = chainedMethod;
      baseClassObject[overrideMarkerName] = this;
      var self = this;
      baseClassObject[methodName] = function() {
        var retValue = self.baseClassMethod.apply(this, arguments);
        return self.chainedMethod.call(subClassObject, this, retValue, arguments);
      }
    }
  }
};

SeleniumExpert.SuiteObserver = function(editor, expert) {
  this.expert = expert;
  editor.app.addObserver(this);
};

SeleniumExpert.SuiteObserver.prototype.testSuiteUnloaded = function(testSuite) {
  this.expert.clear();
};


/**
 * Pane/message related fns
 */
SeleniumExpert.prototype.show = function() {
  this.view.hidden = false;
  document.getElementById("seExpertButtons").hidden = false;
  document.getElementById("seExpertTab").setAttribute("selected", "true");
  document.getElementById("fbarSeExpert").open(0);
};

SeleniumExpert.prototype.hide = function() {
  document.getElementById("fbarSeExpert").close();
  document.getElementById("seExpertTab").removeAttribute("selected");
  this.view.hidden = true;
  document.getElementById("seExpertButtons").hidden = true;
};

SeleniumExpert.prototype.clearPane = function() {
  this.categoriesElement.innerHTML = '';
  this.msgElement.innerHTML = '';
  this.msgElement.className = 'no-msg';
};

SeleniumExpert.prototype.insertInspectionCategory = function(inspectorId, inspectorTitle, summary, canFix, isChecked) {
  var li = this.viewDoc.createElement('li');
  li.id = "cat" + inspectorId;
  li.className = "category";
  li.innerHTML = '<span class="category-plus">-</span><span class="category-checkbox"><input id="cat' + inspectorId + '-check" type="checkbox" ' + (canFix ? (isChecked ? 'checked="checked"' : '') : 'disabled="disabled"') + ' value="' + inspectorId + '"/>' + inspectorTitle + '</span><span class="category-summary"> ' + (canFix ? '' : '(Should be fixed manually)') + (summary ? summary : '') + '</span><ul id="cat' + inspectorId + '-items" class="items"></ul>';
  this.categoriesElement.appendChild(li);
  return this.viewDoc.getElementById("cat" + inspectorId + "-items");
};

SeleniumExpert.prototype.insertInspectionMsg = function(itemList, inspectorId, id, inspectionMsg, statusMsg) {
  var li = this.viewDoc.createElement('li');
  li.value = id;
  /*if (applyOddClass) {
   li.className = "odd";
   }*/
  li.appendChild(this.viewDoc.createTextNode(inspectionMsg));
  /*
   var a = this.viewDoc.createElement('a');
   a.className = 'item-view';
   a.href = "#"
   a.addEventListener("click", function() {window.editor.seExpertView.showCommand(inspectorId, id); return false;}, false);
   a.appendChild(this.viewDoc.createTextNode("View"));
   li.appendChild(a);
   */
  if (statusMsg) {
    var span = this.viewDoc.createElement('span');
    span.className = 'item-status';
    span.appendChild(this.viewDoc.createTextNode(" (" + statusMsg + ")"));
    li.appendChild(span);
  }
  itemList.appendChild(li);
};

SeleniumExpert.prototype.showMessage = function(message) {
  this.msgElement.innerHTML = message;
  this.msgElement.className = 'msg';
};


/**
 * Util fns for all inspectors
 */
SeleniumExpert.AttributeInspectorUtil = {};
SeleniumExpert.AttributeInspectorUtil.getAttributeName = function(target) {
  if (target) {
    var attrPos = target.lastIndexOf("@");
    if (attrPos > 0) {
      return target.substring(attrPos + 1).toLowerCase();
    }
  }
  return null;
};

SeleniumExpert.AttributeInspectorUtil.BOOLEAN_ATTRIBUTES_ = {
  'async': true,
  'autofocus': true,
  'autoplay': true,
  'checked': true,
  'compact': true,
  'complete': true,
  'controls': true,
  'declare': true,
  'defaultchecked': true,
  'defaultselected': true,
  'defer': true,
  'disabled': true,
  'draggable': true,
  'ended': true,
  'formnovalidate': true,
  'hidden': true,
  'indeterminate': true,
  'iscontenteditable': true,
  'ismap': true,
  'itemscope': true,
  'loop': true,
  'multiple': true,
  'muted': true,
  'nohref': true,
  'noresize': true,
  'noshade': true,
  'novalidate': true,
  'nowrap': true,
  'open': true,
  'paused': true,
  'pubdate': true,
  'readonly': true,
  'required': true,
  'reversed': true,
  'scoped': true,
  'seamless': true,
  'seeking': true,
  'selected': true,
  'spellcheck': true,
  'truespeed': true,
  'willvalidate': true
};


/**
 * The Inspectors
 */
function BooleanAttributeInspector() {
  this.name = "BooleanAttributeInspector";
  this.description = "In Selenium IDE v1.0.10 onwards boolean attributes should be true or false";
  this.version = "1.0";
}

BooleanAttributeInspector.prototype.inspect = function(cmd) {
  var attrName = SeleniumExpert.AttributeInspectorUtil.getAttributeName(cmd.target);
  if (attrName) {
    if (SeleniumExpert.AttributeInspectorUtil.BOOLEAN_ATTRIBUTES_[attrName]) {
      if (cmd.value && cmd.value.toLowerCase() == attrName) {
        return "Boolean attribute '" + attrName + "' in '" + cmd.command + "' command should be true.";
      }
      //TODO: detect variables in target and value
    }
  }
  return null;
};

BooleanAttributeInspector.prototype.fix = function(cmd) {
  //Always returns null (if command was not fixed) or an array of zero or more commands to replace the current command
  // try to return the same command so that other properties will remain the same.
  if (this.inspect(cmd)) {
    var msg = "Changed value from '" + cmd.value + "' to 'true'.";
    cmd.value = 'true';
    return [msg, cmd];
  }
  return null;
};


function StyleAttributeLowerCaseInspector() {
  this.name = "StyleAttributeLowerCaseInspector";
  this.description = "In Selenium IDE v1.0.10 onwards style attributes always return lower case values";
  this.version = "1.0";
}

StyleAttributeLowerCaseInspector.prototype.inspect = function(cmd) {
  var attrName = SeleniumExpert.AttributeInspectorUtil.getAttributeName(cmd.target);
  if (attrName) {
    if (attrName == "style") {
      if (cmd.value && cmd.value.toLowerCase() != cmd.value) {
        return "Style attribute value in '" + cmd.command + "' command should be in lower case.";
      }
      //TODO: detect variables in target and value
    }
  }
  return null;
};

StyleAttributeLowerCaseInspector.prototype.fix = function(cmd) {
  //Always returns null (if command was not fixed) or an array of zero or more commands to replace the current command
  // try to return the same command so that other properties will remain the same.
  if (this.inspect(cmd)) {
    var msg = "Changed value from '" + cmd.value + "' to '" + cmd.value.toLowerCase() + "'.";
    cmd.value = cmd.value.toLowerCase();
    return [msg, cmd];
  }
  return null;
};


function UnsupportedSizzleSelectorsInspector() {
  this.name = "UnsupportedSizzleSelectorsInspector";
  this.description = "In Selenium IDE v1.0.8 onwards some CSS selectors are not supported.";
  this.version = "0.1";
}

UnsupportedSizzleSelectorsInspector.UnsupportedSelectors = [
  ':root',
  ':target',
  ':nth-last-child(',
  ':nth-of-type(',
  ':nth-last-of-type(',
  ':first-of-type',
  ':last-of-type',
  ':only-of-type',
  ':lang('
];

UnsupportedSizzleSelectorsInspector.prototype.inspect = function(cmd) {
  //TODO: Determine the right place for a css locator based on the command?
  //TODO: which commands to run for this inspection? which to exclude?
  if (cmd.target && cmd.target.indexOf('css=') == 0) {
    for (var i = 0; i < UnsupportedSizzleSelectorsInspector.UnsupportedSelectors.length; i++) {
      if (cmd.target.indexOf(UnsupportedSizzleSelectorsInspector.UnsupportedSelectors[i]) > 0) {
        return "CSS selector '" + UnsupportedSizzleSelectorsInspector.UnsupportedSelectors[i] + "' in '" + cmd.command + "' command is not supported.";
      }
      //TODO: detect variables in target and value
    }
  }
  if (cmd.value && cmd.value.indexOf('css=') == 0) {
    for (var i = 0; i < UnsupportedSizzleSelectorsInspector.UnsupportedSelectors.length; i++) {
      if (cmd.value.indexOf(UnsupportedSizzleSelectorsInspector.UnsupportedSelectors[i]) > 0) {
        return "CSS selector '" + UnsupportedSizzleSelectorsInspector.UnsupportedSelectors[i] + "' in '" + cmd.command + "' command is not supported.";
      }
      //TODO: detect variables in target and value
    }
  }
  return null;
};


function StoreArgumentsWrongInspector() {
  this.name = "StoreArgumentsWrongInspector";
  this.description = "The arguments to the store* commands can be in the wrong order";
  this.version = "1.0";
}

StoreArgumentsWrongInspector.isVarRegExp = /^[0-9A-Z_]+$/i;
StoreArgumentsWrongInspector.isPossiblyVariable = function(varName) {
  return StoreArgumentsWrongInspector.isVarRegExp.test(varName);
};

StoreArgumentsWrongInspector.prototype.inspect = function(cmd) {
  if (cmd.target) {
    if (cmd.value) {
      if (StoreArgumentsWrongInspector.isPossiblyVariable(cmd.target) && !StoreArgumentsWrongInspector.isPossiblyVariable(cmd.value)) {
        return "Arguments to the '" + cmd.command + "' command seem to be interchanged.";
      }
    }
  }
  return null;
};

StoreArgumentsWrongInspector.prototype.fix = function(cmd) {
  //Always returns null (if command was not fixed) or an array of zero or more commands to replace the current command
  // try to return the same command so that other properties will remain the same.
  if (this.inspect(cmd)) {
    var msg = "Interchanged target and value arguments.";
    var target = cmd.target;
    cmd.target = cmd.value;
    cmd.value = target;
    return [msg, cmd];
  }
  return null;
};


/**
 * The main init code
 */
try {
  this.editor.seExpertView = new SeleniumExpert(this.editor);
  this.editor.seExpertView.registerCommandInspector(new BooleanAttributeInspector(), '(verify|assert)Attribute');
  this.editor.seExpertView.registerCommandInspector(new StyleAttributeLowerCaseInspector(), '(verify|assert)(Not)?Attribute');
  this.editor.seExpertView.registerCommandInspector(new UnsupportedSizzleSelectorsInspector());
  this.editor.seExpertView.registerCommandInspector(new StoreArgumentsWrongInspector(), 'store(Eval|ElementPresent)?$');
} catch (error) {
  alert('Error in SeleniumExpert: ' + error);
}